from .config import LoggerConfig
from .models import LoggingLevel


class Logger:
    config = LoggerConfig()

    def __init__(self):
        self.file_name = None
        self._interface = _LoggerInterface(logger=self)

    def get_logger(self, file_name) -> "_LoggerInterface":
        self.file_name = file_name
        return self._interface


class _LoggerInterface:
    def __init__(self, logger):
        self._logger = logger

    @property
    def config(self):
        return self._logger.config

    @property
    def file_name(self):
        return self._logger.file_name

    def basic_config(self, **kwargs):
        for field, value in kwargs.items():
            if not hasattr(self._logger.config, field):
                raise AttributeError(
                    "No {field} in config of logger.".format(field=field)
                )

            if field == "level":
                if not isinstance(value, str):
                    raise AttributeError("Level should be provided as string.")

                value = getattr(LoggingLevel, value)

            setattr(self._logger.config, field, value)

    def set_stream(self, stream: str, type_):
        self.process_handlers()

        if stream[-1] != "/":
            stream += "/"

        for idx, handler in enumerate(self._logger.config.handlers):
            if not isinstance(handler, type_):
                continue

            if handler.__class__.__name__ == "TelegramHandler":
                self._logger.config.handlers[idx].token = stream

            self._logger.config.handlers[idx].stream = stream

    def process_handlers(self) -> None:
        for idx, handler in enumerate(self._logger.config.handlers):
            if not callable(handler):
                continue

            self._logger.config.handlers[idx] = handler(logger=self)

    def check_handlers(self):
        for idx, handler in enumerate(self._logger.config.handlers):
            if callable(handler):
                handler = handler(logger=self)
                self._logger.config.handlers[idx] = handler

            yield handler

    def info(self, msg):
        for handler in self.check_handlers():
            handler.info(msg=msg)

    def debug(self, msg):
        for handler in self.check_handlers():
            handler.debug(msg=msg)

    def warning(self, msg):
        for handler in self.check_handlers():
            handler.warning(msg=msg)

    def error(self, msg):
        for handler in self.check_handlers():
            handler.error(msg=msg)

    def critical(self, msg):
        for handler in self.check_handlers():
            handler.critical(msg=msg)



import asyncio
from datetime import datetime
from inspect import getframeinfo, stack

import aiohttp

from sherlogger.handlers import AbstractHandler
from sherlogger.logger import Logger
from sherlogger.models import BColors, LoggingLevel
from sherlogger.plugins.config import load_config

from .executor import run_in_separated_thread


class TelegramHandler(AbstractHandler):
    def __init__(self, logger: Logger):
        # Define logger
        self._logger = logger

        # Load configs of telegram bot
        self.config = load_config(self._logger.config.plugins_ini_path)
        self.chat_ids = self.config.telegram_bot.chat_ids
        self.token = self.config.telegram_bot.token

    def _init_msg(self, msg, lineno, level):
        now_time = datetime.utcnow()
        file_name = self._logger.file_name
        msg_format = self._logger.config.formatter

        return msg_format.format(
            asctime=now_time,
            filename=file_name,
            lineno=lineno,
            levelname=level,
            message=msg,
        )

    async def _post(self, msg):
        for chat_id in self.chat_ids:
            payload = {"chat_id": chat_id, "text": msg, "parse_mode": "HTML"}
            url = "https://api.telegram.org/bot{api_token}/sendMessage".format(
                api_token=self.token
            )

            async with aiohttp.ClientSession() as session:
                async with session.post(url, data=payload) as _:
                    await asyncio.sleep(0)

    def info(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.INFO.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        message = self._init_msg(msg=msg, lineno=caller.lineno, level=LoggingLevel.INFO)
        print(BColors.OKBLUE.value + message + BColors.ENDC.value)

        run_in_separated_thread(self._post, message)

    def debug(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.DEBUG.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        message = self._init_msg(
            msg=msg, lineno=caller.lineno, level=LoggingLevel.DEBUG
        )
        print(BColors.OKCYAN.value + message + BColors.ENDC.value)

        run_in_separated_thread(self._post, message)

    def warning(self, msg) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.WARNING.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        message = self._init_msg(
            msg=msg, lineno=caller.lineno, level=LoggingLevel.WARNING
        )
        print(BColors.WARNING.value + message + BColors.ENDC.value)

        run_in_separated_thread(self._post, message)

    def error(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.ERROR.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        message = self._init_msg(
            msg=msg, lineno=caller.lineno, level=LoggingLevel.ERROR
        )
        print(BColors.FAIL.value + message + BColors.ENDC.value)

        run_in_separated_thread(self._post, message)

    def critical(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.CRITICAL.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        message = self._init_msg(
            msg=msg, lineno=caller.lineno, level=LoggingLevel.CRITICAL
        )
        print(BColors.FAIL.value + message + BColors.ENDC.value)

        run_in_separated_thread(self._post, message)



from abc import ABC, abstractmethod
from datetime import datetime
from inspect import getframeinfo, stack

from .logger import Logger
from .models import BColors, LoggingLevel


class AbstractHandler(ABC):
    @abstractmethod
    def info(self, msg: str) -> None:
        pass

    @abstractmethod
    def debug(self, msg: str) -> None:
        pass

    @abstractmethod
    def warning(self, msg: str) -> None:
        pass

    @abstractmethod
    def error(self, msg: str) -> None:
        pass

    @abstractmethod
    def critical(self, msg: str) -> None:
        pass


class FileSystemHandler(AbstractHandler):
    def __init__(self, logger: Logger) -> None:
        self.stream = None
        self._logger = logger

    def _init_msg(self, msg: str, lineno: int, level):
        now_time = datetime.utcnow()
        file_name = self._logger.file_name
        msg_format = self._logger.config.formatter

        return msg_format.format(
            asctime=now_time,
            filename=file_name,
            lineno=lineno,
            levelname=level,
            message=msg,
        )

    def _write(self, msg: str, lineno: int, color, level):
        message = self._init_msg(msg=msg, lineno=lineno, level=level)
        if self.stream is None:
            print(color.value + message + BColors.ENDC.value)
            return

        if level.value[1] != "INFO":
            with open(
                self.stream + LoggingLevel.INFO.value[1].lower() + ".log", "a+"
            ) as f:
                f.write(message + "\n")

        with open(self.stream + level.value[1].lower() + ".log", "a+") as f:
            f.write(message + "\n")
            print(color.value + message + BColors.ENDC.value)

    def info(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.INFO.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        self._write(
            msg=msg, lineno=caller.lineno, color=BColors.OKBLUE, level=LoggingLevel.INFO
        )

    def debug(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.DEBUG.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        self._write(
            msg=msg,
            lineno=caller.lineno,
            color=BColors.OKCYAN,
            level=LoggingLevel.DEBUG,
        )

    def warning(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.WARNING.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        self._write(
            msg=msg,
            lineno=caller.lineno,
            color=BColors.WARNING,
            level=LoggingLevel.WARNING,
        )

    def error(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.ERROR.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        self._write(
            msg=msg, lineno=caller.lineno, color=BColors.FAIL, level=LoggingLevel.ERROR
        )

    def critical(self, msg: str) -> None:
        if self._logger.config.level.value[0] > LoggingLevel.CRITICAL.value[0]:
            return

        caller = getframeinfo(stack()[-1][0])
        self._write(
            msg=msg,
            lineno=caller.lineno,
            color=BColors.FAIL,
            level=LoggingLevel.CRITICAL,
        )



from typing import List, Type, TypeVar, Union

from .models import LoggingLevel, WriteFormat

T = TypeVar("T")


class LoggerConfig:
    level: LoggingLevel = LoggingLevel.NOTSET
    formatter: str = WriteFormat.FORMAT
    handlers: List[Type[T]] = []
    propagate: bool = False
    plugins_ini_path = None

    def to_dict(self) -> dict:
        return {
            "LEVEL": self.level,
            "FORMAT": self.formatter,
            "HANDLERS": self.handlers,
        }

    def set_handlers(self, handlers):
        self.handlers = handlers

    def set_level(self, level: Union[int, str]):
        if isinstance(level, str):
            self.level = getattr(LoggingLevel, level.upper())
        else:
            if level == 0:
                self.level = LoggingLevel.NOTSET
            elif level == 10:
                self.level = LoggingLevel.DEBUG
            elif level == 20:
                self.level = LoggingLevel.INFO
            elif level == 30:
                self.level = LoggingLevel.WARNING
            elif level == 40:
                self.level = LoggingLevel.ERROR
            elif level == 50:
                self.level = LoggingLevel.CRITICAL

    def set_format(self, format_: str):
        WriteFormat.FORMAT = format_
        self.formatter = WriteFormat.FORMAT



import asyncio
import threading


async def do_task(func, message):
    await asyncio.gather(func(message))


def run_in_separated_thread(func, message):
    threads = threading.Thread(target=asyncio.run, args=(do_task(func, message),))
    threads.start()



import configparser
from dataclasses import dataclass


@dataclass
class TelegramBot:
    token: str
    chat_ids: list


@dataclass
class Config:
    telegram_bot: TelegramBot


def load_config(path: str):
    config = configparser.ConfigParser()
    config.read(path)
    print(path)
    tg_bot = config["tg_bot"]

    return Config(
        telegram_bot=TelegramBot(
            token=tg_bot["token"],
            chat_ids=tg_bot["chat_ids"].split(",")
            if "," in tg_bot["chat_ids"]
            else [tg_bot["chat_ids"]],
        ),
    )



from dataclasses import dataclass
from enum import Enum


@dataclass
class WriteFormat:
    FORMAT = "{asctime} - {filename}:{lineno} - {levelname} - {message}"


class BColors(Enum):
    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKCYAN = "\033[96m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


class LoggingLevel(Enum):
    CRITICAL = (50, "CRITICAL")
    ERROR = (40, "ERROR")
    WARNING = (30, "WARNING")
    INFO = (20, "INFO")
    DEBUG = (10, "DEBUG")
    NOTSET = (0, "NOTSET")



from inspect import getmodule, stack

from .handlers import FileSystemHandler
from .logger import Logger, LoggerConfig
from .plugins.telegram.handlers import TelegramHandler

_frame = stack()[-1]
_module = getmodule(_frame[0])

_module_name = "__main__"
if _module is not None:
    _module_name = _module.__file__.split("/")[-1]

sherlog = Logger()

logger = sherlog.get_logger(_module_name)
logger.basic_config(handlers=[FileSystemHandler], level="INFO")
logger.set_stream("./", FileSystemHandler)


def get_telegram_logger(filename: str, ini_file_path: str):
    _tlogger = sherlog.get_logger(filename)
    _tlogger.basic_config(
        handlers=[TelegramHandler], level="INFO", plugins_ini_path=ini_file_path
    )
    return _tlogger


__all__ = [
    "logger",
    "sherlog",
    "LoggerConfig",
    "FileSystemHandler",
    "get_telegram_logger",
]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sherlogger",
    version="0.0.3",
    author="Kel0",
    author_email="rozovdima123@gmail.com",
    description="Simple logger which can work with network!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Kel0/sherlog",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=[
        "aiohttp==3.7.4.post0"
    ]
)
